"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;
exports.useSocketContext = useSocketContext;
var _react = _interopRequireWildcard(require("react"));
var _reactNative = require("react-native");
var _Context = require("./Context");
var _Message = require("../../types/Message");
var _AsyncStorage = require("../AsyncStorage");
var _ChatConfigurations = require("../ChatConfigurations");
var _getHourAndMinutes = require("../../utils/getHourAndMinutes");
var _constants = require("../../constants");
var _api = _interopRequireDefault(require("../../services/api"));
var _useDebounce = require("../../hooks/useDebounce");
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }
function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }
function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }
const botChannel = 'WebChat';
const TWO_SECONDS = 2000;
function SocketContextComponent(_ref) {
  let {
    children,
    botId,
    params
  } = _ref;
  const clientRef = (0, _react.useRef)(null);
  const isFirstRender = (0, _react.useRef)(true);
  const {
    getItemsAsync,
    clearAsync,
    saveDataAsync
  } = (0, _AsyncStorage.useAsyncStorage)();
  const {
    fetchBotAndUpdateConfigs
  } = (0, _ChatConfigurations.useChatConfigurations)();
  const [storageState, setStorageState] = (0, _react.useState)(null);
  const debouncedMessages = (0, _useDebounce.useDebounce)(storageState === null || storageState === void 0 ? void 0 : storageState.messages, TWO_SECONDS);
  const subscribe = async () => {
    const {
      data
    } = await _api.default.post('/webchat/subscribe', {
      sessionId: storageState === null || storageState === void 0 ? void 0 : storageState.sessionId,
      botId,
      channel: 'webchat',
      botChannel,
      parameters: JSON.stringify(params || '[]')
    });
    return data;
  };
  const addMessage = newMessage => {
    setStorageState(prevState => {
      var _prevState$messages, _prevState$messages$f;
      const newMessages = [...((prevState === null || prevState === void 0 ? void 0 : (_prevState$messages = prevState.messages) === null || _prevState$messages === void 0 ? void 0 : (_prevState$messages$f = _prevState$messages.filter) === null || _prevState$messages$f === void 0 ? void 0 : _prevState$messages$f.call(_prevState$messages, m => m.type !== _Message.MessageTypes.TYPING)) ?? []), {
        ...newMessage,
        hour: (0, _getHourAndMinutes.getHourAndMinutes)()
      }];
      return {
        messages: newMessages,
        sessionId: prevState === null || prevState === void 0 ? void 0 : prevState.sessionId
      };
    });
  };
  const handleSubmitMessage = (0, _react.useCallback)(async _ref2 => {
    let {
      message,
      type,
      ext
    } = _ref2;
    const isMedia = type !== _Message.MessageTypes.TEXT;
    try {
      addMessage({
        from: _Message.From.USER,
        message,
        type
      });
      await _api.default.post('/webchat/message', {
        botId,
        message,
        isMedia,
        ext,
        sessionId: storageState === null || storageState === void 0 ? void 0 : storageState.sessionId,
        botChannel,
        isPreview: false
      });
    } catch (error) {
      console.log('Error sending message', error);
    }
  }, [storageState === null || storageState === void 0 ? void 0 : storageState.sessionId]);
  const onClientMessage = msg => {
    addMessage({
      ...msg,
      from: _Message.From.BOT
    });
  };
  async function startConversation() {
    const subscribeResponse = await subscribe();
    if (!(storageState !== null && storageState !== void 0 && storageState.sessionId) || subscribeResponse.sessionId !== (storageState === null || storageState === void 0 ? void 0 : storageState.sessionId)) {
      const newData = {
        sessionId: subscribeResponse.sessionId,
        messages: (storageState === null || storageState === void 0 ? void 0 : storageState.messages) || []
      };
      setStorageState(newData);
      await saveDataAsync(newData);
    }
    return subscribeResponse;
  }
  const restartConversation = async () => {
    setStorageState(null);
    await clearAsync();
    await startConversation();
  };
  (0, _react.useEffect)(() => {
    (async () => {
      if (debouncedMessages !== null && debouncedMessages !== void 0 && debouncedMessages.length && storageState !== null && storageState !== void 0 && storageState.sessionId) {
        await saveDataAsync(storageState);
      }
    })();
  }, [debouncedMessages]);
  (0, _react.useEffect)(() => {
    getItemsAsync().then(stored => {
      setStorageState(stored);
    }).catch(console.log);
  }, []);
  (0, _react.useEffect)(() => {
    const client = new WebSocket(_constants.SOCKET_URL);
    if (!clientRef.current) {
      clientRef.current = client;
      client.onmessage = msg => {
        try {
          const serverResponse = JSON.parse(msg.data);
          if (serverResponse.action === 'message') {
            onClientMessage(serverResponse.data);
          }
        } catch (error) {
          console.log('Error treating received message', error);
        }
      };
    }
    return () => {
      client.close();
      clientRef.current = null;
    };
  }, [_constants.SOCKET_URL]);
  (0, _react.useEffect)(() => {
    (async () => {
      await fetchBotAndUpdateConfigs(botId);
    })();
  }, []);
  (0, _react.useEffect)(() => {
    if (isFirstRender.current) {
      isFirstRender.current = false;
      return;
    }
    startConversation();
  }, []);
  const handleOpenLink = async url => {
    await _reactNative.Linking.openURL(url);
  };
  const handleCarouselButtonClick = async _ref3 => {
    let {
      clickedButton,
      clickedCard
    } = _ref3;
    const {
      destination
    } = clickedButton;
    if ((destination === null || destination === void 0 ? void 0 : destination.type) === _Message.CarouselDestinationTypes.URL) {
      handleOpenLink(destination === null || destination === void 0 ? void 0 : destination.value);
    }
    if ((destination === null || destination === void 0 ? void 0 : destination.type) === _Message.CarouselDestinationTypes.PHONE) {
      var _destination$value;
      const treated = destination === null || destination === void 0 ? void 0 : (_destination$value = destination.value) === null || _destination$value === void 0 ? void 0 : _destination$value.replace(/[^A-Z0-9]/gi, '');
      handleOpenLink(`tel:+55${treated}`);
    }
    await _api.default.post('/webchat/action', {
      botId: botId,
      action: {
        type: _Message.MessageTypes.CAROUSEL,
        data: {
          clickedButton,
          clickedCard
        }
      },
      sessionId: storageState === null || storageState === void 0 ? void 0 : storageState.sessionId
    });
  };
  return /*#__PURE__*/_react.default.createElement(_Context.SocketContextProvider, {
    value: {
      handleCarouselButtonClick,
      handleSubmitMessage,
      messages: (storageState === null || storageState === void 0 ? void 0 : storageState.messages) || [],
      restartConversation
    }
  }, children);
}
var _default = SocketContextComponent;
exports.default = _default;
function useSocketContext() {
  return (0, _react.useContext)(_Context.SocketContext);
}
//# sourceMappingURL=Component.js.map